_debug("Setting up jV extension stuff.");

function logger(text) {
  // log to console
  var consoleService = Components.classes["@mozilla.org/consoleservice;1"].
    getService(Components.interfaces.nsIConsoleService);
  consoleService.logStringMessage(text);
}

function getMenuItemGlobal() {
  return document.getElementById("jv-menuitem-global");
}

function getMenuItemLocal() {
  return document.getElementById("jv-menuitem-local");
}

function getIcon() {
  return document.getElementById("jv-statusbar-image");
}

// TODO: Do we really need to call this so often [every notifyAll]?
function updatePopup() {
  if (getEnabledPref()) {
    this.getMenuItemGlobal().setAttribute("checked", "true");
    this.getMenuItemLocal().removeAttribute("disabled");
  } else {
    this.getMenuItemGlobal().setAttribute("checked", "false");
    this.getMenuItemLocal().setAttribute("disabled", "true");
  }
  this.getMenuItemLocal().setAttribute("checked", this.lenabled);
}

function updateIcon() {
  var src_active = "chrome://jv/skin/statusbar-active.png";
  var src_inactive = "chrome://jv/skin/statusbar-inactive.png";
  var src_locally_inactive = "chrome://jv/skin/statusbar-locally-inactive.png";

  if (getEnabledPref()) {
    if (this.lenabled) {
      this.getIcon().setAttribute("src", src_active);
    } else {
      this.getIcon().setAttribute("src", src_locally_inactive);
    }
  } else {
    this.getIcon().setAttribute("src", src_inactive);
  }
}

function init() {
  this.updateIcon();
  this.updateEditorPrefs();
}

// Grabs clicks everywhere, ignoring those that hit the textarea, so as to clear
// this.e so that we will know not to grab ESC.
function handleGlobalClick(event) {
  if (!this.isEnabled() || !this.e) {
    return;
  }
  if (this.target != this.e) {
    this.e = null;
  }
}

// Currently this function just catches keypresses that we'd miss due to being
// enabled, but without a handler being attached to this textarea.  Later we
// might want to try to hack it to grab the escape key before GMail gets it.
function handleGlobalKeypress(event) {
  var e = event.originalTarget;
  if (!this.isEnabled()) {
    return;
  }
  var enableFlicker = false;
  if (this.e && !this.getCharCode(event) &&
      this.getKeyCode(event) == this.KeyCodes.ESC) {
    // Grab ESC before GMail gets it by snagging it early here.
    this.handleKeypress(event, enableFlicker, true);
    // This does not, however, prevent GMail from getting it.  Either it gets it
    // after us, but during the non-cancellable capturing phase, or it gets it
    // before us but, similarly, can't prevent us from getting it.  Assuming the
    // latter, calling focus() on our most-recent element should do the trick.
    // TODO: Some way not to always grab all escapes; make it a preference.
    this.e.focus();

    return;
  }

  if (!e || !e.tagName || e.tagName.toLowerCase() != "textarea") {
    return;
  }
  if (e != event.originalTarget) {
    // These hit in GMail navigation sometimes.  In one case, I'd
    // clicked-to-highlight in a textarea that was disallowed.
    return;
  }
  if (event.altKey || event.metaKey) {
    return;
  }
  if (this.isDisallowed(e)) {
    return;
  }
  enableFlicker = true;
  if (this.setUpElementIfNeeded(e, enableFlicker)) {
    // Don't know if that's always soon enough to get this particular keypress
    // event handled properly, so let's just call the handler from here, too.
    // If we start seeing initial key events doubled, this code's what's to
    // blame.
    this.handleKeypress(event, enableFlicker, true);
  }
}

// nsIEventListener implementation
function handleEvent(event) {
  if (event.type == "load") {
    this.init();
  } else if (event.type == "keypress") {
    this.handleGlobalKeypress(event);
  } else if (event.type == "click") {
    this.handleGlobalClick(event);
  } else if (this.isEnabled()) {
    this.jvHandleEvent(event); // Defined in jv.js.
  }
}

function jv_updateIcon() {
  jsvim.updateIcon();
}

function jv_onClickStatusIcon(evt) {
  if (evt.button != 0) {
    return;
  }
  try {
    var wnd = window.openDialog("chrome://jv/content/settings.xul",
                  "jv-settings",
                  "resizable=no");
    wnd.addEventListener("unload", jv_updateIcon, false);
  } catch (ex) {
    popup(stringifyObj(ex));
  }
}

function jv_globalEnable() {
  var enabled = !getEnabledPref();
  saveEnabledPref(enabled);
  notifyAll(enabled);
}

function jv_localEnable() {
  if (jsvim.getMenuItemLocal().getAttribute("checked") != "true") {
    jsvim.lenabled = false;
  } else {
    jsvim.lenabled = true;
  }
  jsvim.updatePopup();
  jsvim.updateIcon();
}

function jv_onPopupShowing() {
  jsvim.updatePopup();
}

function jv_showAbout() {
  window.openDialog("chrome://jv/content/about.xul",
            "jv-about", "");
}

function canAccessXSelection() {
  var clipsvc = Components.classes["@mozilla.org/widget/clipboard;1"]
    .getService(Components.interfaces.nsIClipboard);
  return clipsvc.supportsSelectionClipboard();
}

function setClipboardText(copytext, which) {
  var clip;
  var str = Components.classes["@mozilla.org/supports-string;1"].
    createInstance(Components.interfaces.nsISupportsString);
  str.data = copytext;

  var trans = Components.classes["@mozilla.org/widget/transferable;1"].
    createInstance(Components.interfaces.nsITransferable);
  trans.addDataFlavor("text/unicode");
  trans.setTransferData("text/unicode", str, copytext.length * 2);
  var clipid = Components.interfaces.nsIClipboard;
  var clipsvc   = Components.classes["@mozilla.org/widget/clipboard;1"]
    .getService(clipid);

  if ( which == 2 && clipsvc.supportsSelectionClipboard() )
    clip = clipid.kSelectionClipboard;
  else
    clip = clipid.kGlobalClipboard;
  clipsvc.setData(trans, null, clip);
}

function getClipboardText(which) {
  var clip;
  var clipid = Components.interfaces.nsIClipboard;
  var clipsvc  = Components.classes["@mozilla.org/widget/clipboard;1"]
    .getService(clipid);
  var trans = Components.classes["@mozilla.org/widget/transferable;1"]
    .createInstance(Components.interfaces.nsITransferable);
  trans.addDataFlavor("text/unicode");

  if ( which == 2 && clipsvc.supportsSelectionClipboard() )
    clip = clipid.kSelectionClipboard;
  else
    clip = clipid.kGlobalClipboard;
  clipsvc.getData(trans, clip);
  var str = new Object();
  var len = new Object();
  trans.getTransferData("text/unicode", str, len);
  if (str)
    str = str.value.
      QueryInterface(Components.interfaces.nsISupportsString);
  if (str)
    return str ? str.data.substring(0, len.value/2) : null;
}

function getXSelection() {
  return getClipboardText(2);
}

function getClipboard() {
  return getClipboardText(1);
}

function setXSelection(text) {
  return setClipboardText(text, 2);
}

function setClipboard(text) {
  return setClipboardText(text, 1);
}

var p = JsVim.prototype;

p.getIcon = getIcon;
p.getMenuItemGlobal = getMenuItemGlobal;
p.getMenuItemLocal = getMenuItemLocal;
p.getPrefs = getPrefs;
p.handleGlobalClick = handleGlobalClick;
p.handleEvent = handleEvent;
p.handleGlobalKeypress = handleGlobalKeypress;
p.inExtension = true; // Store here that we're running in extension context.
p.init = init;
p.isEnabled = isEnabled;
p.updateEditorPrefs = updateEditorPrefs;
p.updateIcon = updateIcon;
p.updatePopup = updatePopup;

if (canAccessXSelection()) {
  p.getXSelection = getXSelection;
  p.setXSelection = setXSelection;
}
p.getClipboard = getClipboard;
p.setClipboard = setClipboard;

_debug("Set up jV extension stuff.");
